@@ -1,18 +1,16 @@
 // SPDX-License-Identifier: GPL-2.0+
 /*
- * Hardware monitoring driver for Microchip EMC2305 fan controller
+ * Hardware monitoring driver for EMC2305 fan controller
  *
- * Copyright (C) 2022 Nvidia Technologies Ltd and Delta Networks, Inc.
+ * Copyright (C) 2022 Nvidia Technologies Ltd.
  */
 
 #include <linux/err.h>
 #include <linux/hwmon.h>
-#include <linux/hwmon-sysfs.h>
 #include <linux/i2c.h>
 #include <linux/module.h>
-#include <linux/of.h>
+#include <linux/platform_data/emc2305.h>
 #include <linux/thermal.h>
-#include <linux/version.h>
 
 static const unsigned short
 emc2305_normal_i2c[] = { 0x27, 0x2c, 0x2d, 0x2e, 0x2f, 0x4c, 0x4d, I2C_CLIENT_END };
@@ -27,24 +25,24 @@ emc2305_normal_i2c[] = { 0x27, 0x2c, 0x2d, 0x2e, 0x2f, 0x4c, 0x4d, I2C_CLIENT_EN
 #define EMC2305_VENDOR			0x5d
 #define EMC2305_REG_PRODUCT_ID		0xfd
 #define EMC2305_TACH_REGS_UNUSE_BITS	3
-#define EMC2305_TACH_CNT_MULTIPLIER	1	/* Set by FAN_CFG RNGx bits */
-#define EMC2305_PWM_MAX			5
-#define EMC2305_PWM_CHNL_CMN		0
+#define EMC2305_TACH_CNT_MULTIPLIER	0x02
+#define EMC2305_TACH_RANGE_MIN		480
 
 #define EMC2305_PWM_DUTY2STATE(duty, max_state, pwm_max) \
-	(DIV_ROUND_CLOSEST((duty) * (max_state), (pwm_max)))
+	DIV_ROUND_CLOSEST((duty) * (max_state), (pwm_max))
 #define EMC2305_PWM_STATE2DUTY(state, max_state, pwm_max) \
-	(DIV_ROUND_CLOSEST((state) * (pwm_max), (max_state)))
+	DIV_ROUND_CLOSEST((state) * (pwm_max), (max_state))
 
-/* Factor by equations [2] and [3] from data sheet; valid for fans where the number of edges
+/*
+ * Factor by equations [2] and [3] from data sheet; valid for fans where the number of edges
  * equal (poles * 2 + 1).
  */
 #define EMC2305_RPM_FACTOR		3932160
 
-#define EMC2305_REG_FAN_DRIVE(n) (0x30 + 0x10 * (n))
-#define EMC2305_REG_FAN_CFG(n) (0x32 + 0x10 * (n))
-#define EMC2305_REG_FAN_MIN_DRIVE(n) (0x38 + 0x10 * (n))
-#define EMC2305_REG_FAN_TACH(n) (0x3e + 0x10 * (n))
+#define EMC2305_REG_FAN_DRIVE(n)	(0x30 + 0x10 * (n))
+#define EMC2305_REG_FAN_CFG(n)		(0x32 + 0x10 * (n))
+#define EMC2305_REG_FAN_MIN_DRIVE(n)	(0x38 + 0x10 * (n))
+#define EMC2305_REG_FAN_TACH(n)		(0x3e + 0x10 * (n))
 
 enum emc230x_product_id {
 	EMC2305 = 0x34,
@@ -64,6 +62,8 @@ MODULE_DEVICE_TABLE(i2c, emc2305_ids);
 
 static const struct of_device_id emc2305_dt_ids[] = {
 	{ .compatible = "microchip,emc2305" },
+	{ .compatible = "microchip,emc2303" },
+	{ .compatible = "microchip,emc2302" },
 	{ .compatible = "microchip,emc2301" },
 	{ }
 };
@@ -100,19 +100,19 @@ struct emc2305_cdev_data {
  * @client: i2c client;
  * @hwmon_dev: hwmon device;
  * @max_state: maximum cooling state of the cooling device;
- * @pwm_max: maximum PWM;
- * @pwm_min: minimum PWM;
- * @pwm_channel: maximum number of PWM channels;
+ * @pwm_num: number of PWM channels;
+ * @pwm_separate: separate PWM settings for every channel;
+ * @pwm_min: array of minimum PWM per channel;
  * @cdev_data: array of cooling devices data;
  */
 struct emc2305_data {
 	struct i2c_client *client;
 	struct device *hwmon_dev;
 	u8 max_state;
-	u8 pwm_max;
-	u8 pwm_min;
 	u8 pwm_num;
-	u8 pwm_channel;
+	bool pwm_separate;
+	u8 pwm_min[EMC2305_PWM_MAX];
+	u8 pwm_max;
 	struct emc2305_cdev_data cdev_data[EMC2305_PWM_MAX];
 };
 
@@ -125,12 +125,11 @@ static char *emc2305_fan_name[] = {
 	"emc2305_fan5",
 };
 
-static int emc2305_get_max_channel(struct emc2305_data *data)
+static void emc2305_unset_tz(struct device *dev);
+
+static int emc2305_get_max_channel(const struct emc2305_data *data)
 {
-	if (data->pwm_channel == EMC2305_PWM_CHNL_CMN)
-		return data->pwm_num;
-	else
-		return data->pwm_channel;
+	return data->pwm_num;
 }
 
 static int emc2305_get_cdev_idx(struct thermal_cooling_device *cdev)
@@ -142,15 +141,16 @@ static int emc2305_get_cdev_idx(struct thermal_cooling_device *cdev)
 	if (len <= 0)
 		return -EINVAL;
 
-	/* Retuns index of cooling device 0..4 in case of separate PWM setting.
+	/*
+	 * Returns index of cooling device 0..4 in case of separate PWM setting.
 	 * Zero index is used in case of one common PWM setting.
-	 * If the mode is set as EMC2305_PWM_CHNL_CMN, all PWMs are to be bound
+	 * If the mode is not set as pwm_separate, all PWMs are to be bound
 	 * to the common thermal zone and should work at the same speed
 	 * to perform cooling for the same thermal junction.
 	 * Otherwise, return specific channel that will be used in bound
 	 * related PWM to the thermal zone.
 	 */
-	if (data->pwm_channel == EMC2305_PWM_CHNL_CMN)
+	if (!data->pwm_separate)
 		return 0;
 
 	ret = cdev->type[len - 1];
@@ -183,13 +183,41 @@ static int emc2305_get_max_state(struct thermal_cooling_device *cdev, unsigned l
 	return 0;
 }
 
-static int emc2305_set_cur_state(struct thermal_cooling_device *cdev, unsigned long state)
+static int __emc2305_set_cur_state(struct emc2305_data *data, int cdev_idx, unsigned long state)
 {
-	int cdev_idx;
-	struct emc2305_data *data = cdev->devdata;
+	int ret;
 	struct i2c_client *client = data->client;
 	u8 val, i;
 
+	state = max_t(unsigned long, state, data->cdev_data[cdev_idx].last_hwmon_state);
+
+	val = EMC2305_PWM_STATE2DUTY(state, data->max_state, EMC2305_FAN_MAX);
+
+	data->cdev_data[cdev_idx].cur_state = state;
+	if (data->pwm_separate) {
+		ret = i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_DRIVE(cdev_idx), val);
+		if (ret < 0)
+			return ret;
+	} else {
+		/*
+		 * Set the same PWM value in all channels
+		 * if common PWM channel is used.
+		 */
+		for (i = 0; i < data->pwm_num; i++) {
+			ret = i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_DRIVE(i), val);
+			if (ret < 0)
+				return ret;
+		}
+	}
+
+	return 0;
+}
+
+static int emc2305_set_cur_state(struct thermal_cooling_device *cdev, unsigned long state)
+{
+	int cdev_idx, ret;
+	struct emc2305_data *data = cdev->devdata;
+
 	if (state > data->max_state)
 		return -EINVAL;
 
@@ -199,22 +227,9 @@ static int emc2305_set_cur_state(struct thermal_cooling_device *cdev, unsigned l
 
 	/* Save thermal state. */
 	data->cdev_data[cdev_idx].last_thermal_state = state;
-	state = max_t(unsigned long, state, data->cdev_data[cdev_idx].last_hwmon_state);
-
-	val = EMC2305_PWM_STATE2DUTY(state, data->max_state, data->pwm_max);
-
-	if (val > EMC2305_FAN_MAX)
-		return -EINVAL;
-
-	data->cdev_data[cdev_idx].cur_state = state;
-	if (data->pwm_channel == EMC2305_PWM_CHNL_CMN)
-	/* Set the same PWM value in all channels
-	 * if common PWM channel is used.
-	 */
-		for (i = 0; i < data->pwm_num; i++)
-			i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_DRIVE(i), val);
-	else
-		i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_DRIVE(cdev_idx), val);
+	ret = __emc2305_set_cur_state(data, cdev_idx, state);
+	if (ret < 0)
+		return ret;
 
 	return 0;
 }
@@ -232,6 +247,9 @@ static int emc2305_show_fault(struct device *dev, int channel)
 	int status_reg;
 
 	status_reg = i2c_smbus_read_byte_data(client, EMC2305_REG_DRIVE_FAIL_STATUS);
+	if (status_reg < 0)
+		return status_reg;
+
 	return status_reg & (1 << channel) ? 1 : 0;
 }
 
@@ -242,11 +260,15 @@ static int emc2305_show_fan(struct device *dev, int channel)
 	int ret;
 
 	ret = i2c_smbus_read_word_swapped(client, EMC2305_REG_FAN_TACH(channel));
-	if (ret < 0)
+	if (ret <= 0)
 		return ret;
 
 	ret = ret >> EMC2305_TACH_REGS_UNUSE_BITS;
-	return EMC2305_RPM_FACTOR * EMC2305_TACH_CNT_MULTIPLIER / (ret > 0 ? ret : 1);
+	ret = EMC2305_RPM_FACTOR / ret;
+	if (ret <= EMC2305_TACH_RANGE_MIN)
+		return 0;
+
+	return ret * EMC2305_TACH_CNT_MULTIPLIER;
 }
 
 static int emc2305_show_pwm(struct device *dev, int channel)
@@ -261,13 +283,16 @@ static int emc2305_set_pwm(struct device *dev, long val, int channel)
 {
 	struct emc2305_data *data = dev_get_drvdata(dev);
 	struct i2c_client *client = data->client;
+	int ret;
 
-	if (val < data->pwm_min || val > data->pwm_max)
+	if (val < data->pwm_min[channel] || val > data->pwm_max)
 		return -EINVAL;
 
-	i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_DRIVE(channel), val);
+	ret = i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_DRIVE(channel), val);
+	if (ret < 0)
+		return ret;
 	data->cdev_data[channel].cur_state = EMC2305_PWM_DUTY2STATE(val, data->max_state,
-								    data->pwm_max);
+								    EMC2305_FAN_MAX);
 	return 0;
 }
 
@@ -276,86 +301,80 @@ static int emc2305_get_tz_of(struct device *dev)
 	struct device_node *np = dev->of_node;
 	struct emc2305_data *data = dev_get_drvdata(dev);
 	int ret = 0;
+	u32 val;
+	int i;
 
 	/* OF parameters are optional - overwrite default setting
 	 * if some of them are provided.
 	 */
 
-	if (of_find_property(np, "emc2305,cooling-levels", NULL)) {
-		ret = of_property_read_u8(np, "emc2305,cooling-levels", &data->max_state);
-		if (ret)
-			return ret;
-	}
+	ret = of_property_read_u32(np, "emc2305,cooling-levels", &val);
+	if (!ret)
+		data->max_state = (u8)val;
+	else if (ret != -EINVAL)
+		return ret;
 
-	if (of_find_property(np, "emc2305,pwm-max", NULL)) {
-		ret = of_property_read_u8(np, "emc2305,pwm-max", &data->pwm_max);
-		if (ret)
-			return ret;
-	}
+	ret = of_property_read_u32(np, "emc2305,pwm-max", &val);
+	if (!ret)
+		data->pwm_max = (u8)val;
+	else if (ret != -EINVAL)
+		return ret;
 
-	if (of_find_property(np, "emc2305,pwm-min", NULL)) {
-		ret = of_property_read_u8(np, "emc2305,pwm-min", &data->pwm_min);
-		if (ret)
-			return ret;
-	}
+	ret = of_property_read_u32(np, "emc2305,pwm-min", &val);
+	if (!ret)
+		for (i = 0; i < EMC2305_PWM_MAX; i++)
+			data->pwm_min[i] = (u8)val;
+	else if (ret != -EINVAL)
+		return ret;
 
 	/* Not defined or 0 means one thermal zone over all cooling devices.
 	 * Otherwise - separated thermal zones for each PWM channel.
 	 */
-	if (of_find_property(np, "emc2305,pwm-channel", NULL)) {
-		ret = of_property_read_u8(np, "emc2305,pwm-channel", &data->pwm_channel);
-		if (ret)
-			return ret;
-	}
+	ret = of_property_read_u32(np, "emc2305,pwm-channel", &val);
+	if (!ret)
+		data->pwm_separate = (val != 0);
+	else if (ret != -EINVAL)
+		return ret;
 
-	return ret;
+	return 0;
 }
 
 static int emc2305_set_single_tz(struct device *dev, int idx)
 {
 	struct emc2305_data *data = dev_get_drvdata(dev);
-	long pwm = data->pwm_max;
-	int cdev_idx;
+	long pwm;
+	int i, cdev_idx, ret;
 
 	cdev_idx = (idx) ? idx - 1 : 0;
+	pwm = data->pwm_min[cdev_idx];
 
-	if (dev->of_node)
-		data->cdev_data[cdev_idx].cdev =
-			devm_thermal_of_cooling_device_register(dev, dev->of_node,
-								emc2305_fan_name[idx], data,
-								&emc2305_cooling_ops);
-	else
-		data->cdev_data[cdev_idx].cdev =
-			thermal_cooling_device_register(emc2305_fan_name[idx], data,
-							&emc2305_cooling_ops);
+	data->cdev_data[cdev_idx].cdev =
+		thermal_cooling_device_register(emc2305_fan_name[idx], data,
+						&emc2305_cooling_ops);
 
 	if (IS_ERR(data->cdev_data[cdev_idx].cdev)) {
 		dev_err(dev, "Failed to register cooling device %s\n", emc2305_fan_name[idx]);
 		return PTR_ERR(data->cdev_data[cdev_idx].cdev);
 	}
-	emc2305_set_pwm(dev, pwm, cdev_idx);
-	data->cdev_data[cdev_idx].cur_state = data->max_state;
 	/* Set minimal PWM speed. */
-	data->cdev_data[cdev_idx].last_hwmon_state = EMC2305_PWM_DUTY2STATE(data->pwm_min,
-									    data->max_state,
-									    data->pwm_max);
-	return 0;
-}
-
-static void emc2305_unset_tz(struct device *dev)
-{
-	struct emc2305_data *data = dev_get_drvdata(dev);
-	int i;
-
-	/* Unregister cooling device in case they have been registered by
-	 * thermal_cooling_device_unregister(). No need for clean-up flow in case they
-	 * have been registered by devm_thermal_of_cooling_device_register()
-	 */
-	if (!dev->of_node) {
-		for (i = 0; i < EMC2305_PWM_MAX; i++)
-			if (data->cdev_data[i].cdev)
-				thermal_cooling_device_unregister(data->cdev_data[i].cdev);
+	if (data->pwm_separate) {
+		ret = emc2305_set_pwm(dev, pwm, cdev_idx);
+		if (ret < 0)
+			return ret;
+	} else {
+		for (i = 0; i < data->pwm_num; i++) {
+			ret = emc2305_set_pwm(dev, pwm, i);
+			if (ret < 0)
+				return ret;
+		}
 	}
+	data->cdev_data[cdev_idx].cur_state =
+		EMC2305_PWM_DUTY2STATE(data->pwm_min[cdev_idx], data->max_state,
+				       EMC2305_FAN_MAX);
+	data->cdev_data[cdev_idx].last_hwmon_state =
+		EMC2305_PWM_DUTY2STATE(data->pwm_min[cdev_idx], data->max_state,
+				       EMC2305_FAN_MAX);
+	return 0;
 }
 
 static int emc2305_set_tz(struct device *dev)
@@ -363,10 +382,10 @@ static int emc2305_set_tz(struct device *dev)
 	struct emc2305_data *data = dev_get_drvdata(dev);
 	int i, ret;
 
-	if (data->pwm_channel == EMC2305_PWM_CHNL_CMN)
+	if (!data->pwm_separate)
 		return emc2305_set_single_tz(dev, 0);
 
-	for (i = 0; i < data->pwm_channel; i++) {
+	for (i = 0; i < data->pwm_num; i++) {
 		ret = emc2305_set_single_tz(dev, i + 1);
 		if (ret)
 			goto thermal_cooling_device_register_fail;
@@ -378,13 +397,24 @@ thermal_cooling_device_register_fail:
 	return ret;
 }
 
+static void emc2305_unset_tz(struct device *dev)
+{
+	struct emc2305_data *data = dev_get_drvdata(dev);
+	int i;
+
+	/* Unregister cooling device. */
+	for (i = 0; i < EMC2305_PWM_MAX; i++)
+		if (data->cdev_data[i].cdev)
+			thermal_cooling_device_unregister(data->cdev_data[i].cdev);
+}
+
 static umode_t
 emc2305_is_visible(const void *data, enum hwmon_sensor_types type, u32 attr, int channel)
 {
-	int max_channel = emc2305_get_max_channel((struct emc2305_data *)data);
+	int max_channel = emc2305_get_max_channel(data);
 
 	/* Don't show channels which are not physically connected. */
-	if ((channel + 1) > max_channel)
+	if (channel >= max_channel)
 		return 0;
 	switch (type) {
 	case hwmon_fan:
@@ -416,6 +446,7 @@ static int
 emc2305_write(struct device *dev, enum hwmon_sensor_types type, u32 attr, int channel, long val)
 {
 	struct emc2305_data *data = dev_get_drvdata(dev);
+	int cdev_idx;
 
 	switch (type) {
 	case hwmon_pwm:
@@ -423,15 +454,21 @@ emc2305_write(struct device *dev, enum hwmon_sensor_types type, u32 attr, int ch
 		case hwmon_pwm_input:
 			/* If thermal is configured - handle PWM limit setting. */
 			if (IS_REACHABLE(CONFIG_THERMAL)) {
-				data->cdev_data[channel].last_hwmon_state =
-					EMC2305_PWM_DUTY2STATE(val, data->max_state, data->pwm_max);
-				/* Update PWM only in case requested state is not less than the
+				if (data->pwm_separate)
+					cdev_idx = channel;
+				else
+					cdev_idx = 0;
+				data->cdev_data[cdev_idx].last_hwmon_state =
+					EMC2305_PWM_DUTY2STATE(val, data->max_state,
+							       EMC2305_FAN_MAX);
+				/*
+				 * Update PWM only in case requested state is not less than the
 				 * last thermal state.
 				 */
-				if (data->cdev_data[channel].last_hwmon_state >=
-				    data->cdev_data[channel].last_thermal_state)
-					return emc2305_set_cur_state(data->cdev_data[channel].cdev,
-							data->cdev_data[channel].last_hwmon_state);
+				if (data->cdev_data[cdev_idx].last_hwmon_state >=
+				    data->cdev_data[cdev_idx].last_thermal_state)
+					return __emc2305_set_cur_state(data, cdev_idx,
+							data->cdev_data[cdev_idx].last_hwmon_state);
 				return 0;
 			}
 			return emc2305_set_pwm(dev, val, channel);
@@ -540,7 +577,7 @@ static int emc2305_identify(struct device *dev)
 		data->pwm_num = 1;
 		break;
 	default:
-		return -EINVAL;
+		return -ENODEV;
 	}
 
 	return 0;
@@ -551,6 +588,7 @@ static int emc2305_probe(struct i2c_client *client, const struct i2c_device_id *
 	struct i2c_adapter *adapter = client->adapter;
 	struct device *dev = &client->dev;
 	struct emc2305_data *data;
+	struct emc2305_platform_data *pdata;
 	int vendor;
 	int ret;
 	int i;
@@ -573,14 +611,34 @@ static int emc2305_probe(struct i2c_client *client, const struct i2c_device_id *
 	if (ret)
 		return ret;
 
-	data->max_state = EMC2305_FAN_MAX_STATE;
-	data->pwm_max = EMC2305_FAN_MAX;
-	data->pwm_min = EMC2305_FAN_MIN;
-	data->pwm_channel = EMC2305_PWM_CHNL_CMN;
-	if (dev->of_node) {
-		ret = emc2305_get_tz_of(dev);
-		if (ret < 0)
-			return ret;
+	pdata = dev_get_platdata(&client->dev);
+	if (pdata) {
+		if (!pdata->max_state || pdata->max_state > EMC2305_FAN_MAX_STATE)
+			return -EINVAL;
+		data->max_state = pdata->max_state;
+		/*
+		 * Validate a number of active PWM channels. Note that
+		 * configured number can be less than the actual maximum
+		 * supported by the device.
+		 */
+		if (!pdata->pwm_num || pdata->pwm_num > EMC2305_PWM_MAX)
+			return -EINVAL;
+		data->pwm_num = pdata->pwm_num;
+		data->pwm_separate = pdata->pwm_separate;
+		for (i = 0; i < EMC2305_PWM_MAX; i++)
+			data->pwm_min[i] = pdata->pwm_min[i];
+		data->pwm_max = EMC2305_FAN_MAX;
+	} else {
+		data->max_state = EMC2305_FAN_MAX_STATE;
+		data->pwm_separate = false;
+		for (i = 0; i < EMC2305_PWM_MAX; i++)
+			data->pwm_min[i] = EMC2305_FAN_MIN;
+		data->pwm_max = EMC2305_FAN_MAX;
+		if (dev->of_node) {
+			ret = emc2305_get_tz_of(dev);
+			if (ret < 0)
+				return ret;
+		}
 	}
 
 	data->hwmon_dev = devm_hwmon_device_register_with_info(dev, "emc2305", data,
@@ -595,15 +653,10 @@ static int emc2305_probe(struct i2c_client *client, const struct i2c_device_id *
 	}
 
 	for (i = 0; i < data->pwm_num; i++) {
-		u8 val;
-
-		i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_MIN_DRIVE(i),
-					  data->pwm_min);
-
-		val = i2c_smbus_read_byte_data(client, EMC2305_REG_FAN_CFG(i));
-		/* Set RNGx so minRPM=500 */
-		val &= ~0x60;
-		i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_CFG(i), val);
+		ret = i2c_smbus_write_byte_data(client, EMC2305_REG_FAN_MIN_DRIVE(i),
+						data->pwm_min[i]);
+		if (ret < 0)
+			return ret;
 	}
 
 	/* Acknowledge any existing faults. Stops the device responding on the
@@ -615,13 +668,12 @@ static int emc2305_probe(struct i2c_client *client, const struct i2c_device_id *
 	return 0;
 }
 
-static int emc2305_remove(struct i2c_client *client)
+static void emc2305_remove(struct i2c_client *client)
 {
 	struct device *dev = &client->dev;
 
 	if (IS_REACHABLE(CONFIG_THERMAL))
 		emc2305_unset_tz(dev);
-	return 0;
 }
 
 static struct i2c_driver emc2305_driver = {
