@@ -66,7 +66,7 @@ struct pivariety {
 	struct v4l2_subdev sd;
 	struct media_pad pad;
 
-	struct v4l2_fwnode_bus_mipi_csi2 bus;
+	struct v4l2_mbus_config_mipi_csi2 bus;
 	struct clk *xclk;
 	u32 xclk_freq;
 
@@ -946,13 +946,13 @@ static int pivariety_get_mbus_config(struct v4l2_subdev *sd, unsigned int pad,
 				     struct v4l2_mbus_config *cfg)
 {
 	struct pivariety *pivariety = to_pivariety(sd);
-	const u32 mask = V4L2_MBUS_CSI2_LANE_MASK;
 
 	if (pivariety->lanes > pivariety->bus.num_data_lanes)
 		return -EINVAL;
 
 	cfg->type = V4L2_MBUS_CSI2_DPHY;
-	cfg->flags = (pivariety->lanes << __ffs(mask)) & mask;
+	cfg->bus.mipi_csi2.flags = pivariety->bus.flags;
+	cfg->bus.mipi_csi2.num_data_lanes = pivariety->lanes;
 
 	return 0;
 }
@@ -1432,7 +1432,7 @@ error_power_off:
 	return ret;
 }
 
-static int pivariety_remove(struct i2c_client *client)
+static void pivariety_remove(struct i2c_client *client)
 {
 	struct v4l2_subdev *sd = i2c_get_clientdata(client);
 	struct pivariety *pivariety = to_pivariety(sd);
@@ -1443,8 +1443,6 @@ static int pivariety_remove(struct i2c_client *client)
 
 	pm_runtime_disable(&client->dev);
 	pm_runtime_set_suspended(&client->dev);
-
-	return 0;
 }
 
 static const struct dev_pm_ops pivariety_pm_ops = {
