@@ -41,14 +41,6 @@ MODULE_PARM_DESC(trigger_mode, "Set vsync trigger mode: 0=standalone, (1=source
 #define OV7251_AEC_EXPO_2		0x3502
 #define OV7251_AEC_AGC_ADJ_0		0x350a
 #define OV7251_AEC_AGC_ADJ_1		0x350b
-/* Exposure must be at least 20 lines shorter than VTS */
-#define OV7251_EXPOSURE_OFFSET		20
- /* HTS is registers 0x380c and 0x380d */
-#define OV7251_HTS			0x3a0
-#define OV7251_VTS_HIGH			0x380e
-#define OV7251_VTS_LOW			0x380f
-#define OV7251_VTS_MIN_OFFSET		92
-#define OV7251_VTS_MAX			0x7fff
 #define OV7251_TIMING_FORMAT1		0x3820
 #define OV7251_TIMING_FORMAT1_VFLIP	BIT(2)
 #define OV7251_TIMING_FORMAT2		0x3821
@@ -66,39 +58,38 @@ MODULE_PARM_DESC(trigger_mode, "Set vsync trigger mode: 0=standalone, (1=source
 #define OV7251_PLL2_SYS_DIV_REG		0x309a
 #define OV7251_PLL2_ADC_DIV_REG		0x309b
 
-/*
- * OV7251 native and active pixel array size.
- * Datasheet not available to confirm these values, so assume there are no
- * border pixels.
- */
-#define OV7251_NATIVE_WIDTH		640U
-#define OV7251_NATIVE_HEIGHT		480U
-#define OV7251_PIXEL_ARRAY_LEFT		0U
-#define OV7251_PIXEL_ARRAY_TOP		0U
-#define OV7251_PIXEL_ARRAY_WIDTH	640U
-#define OV7251_PIXEL_ARRAY_HEIGHT	480U
+#define OV7251_NATIVE_WIDTH		656
+#define OV7251_NATIVE_HEIGHT		496
+#define OV7251_ACTIVE_START_LEFT	4
+#define OV7251_ACTIVE_START_TOP		4
+#define OV7251_ACTIVE_WIDTH		648
+#define OV7251_ACTIVE_HEIGHT		488
 
-#define OV7251_PIXEL_CLOCK 48000000
+#define OV7251_FIXED_PPL		928
+#define OV7251_TIMING_VTS_REG		0x380e
+#define OV7251_TIMING_MIN_VTS		1
+#define OV7251_TIMING_MAX_VTS		0xffff
+#define OV7251_INTEGRATION_MARGIN	20
 
 struct reg_value {
 	u16 reg;
 	u8 val;
 };
 
-struct ov7251_frame_ival_info {
-	u16 vts;
-	struct v4l2_fract timeperframe;
-};
-
 struct ov7251_mode_info {
 	u32 width;
 	u32 height;
+	u32 vts;
 	const struct reg_value *data;
 	u32 data_size;
+	u32 pixel_clock;
+	u32 link_freq;
+	u16 exposure_max;
 	u16 exposure_def;
+	struct v4l2_fract timeperframe;
 };
 
-struct ov7251_pll1_config {
+struct ov7251_pll1_cfg {
 	unsigned int pre_div;
 	unsigned int mult;
 	unsigned int div;
@@ -106,7 +97,7 @@ struct ov7251_pll1_config {
 	unsigned int mipi_div;
 };
 
-struct ov7251_pll2_config {
+struct ov7251_pll2_cfg {
 	unsigned int pre_div;
 	unsigned int mult;
 	unsigned int div;
@@ -114,9 +105,25 @@ struct ov7251_pll2_config {
 	unsigned int adc_div;
 };
 
-struct ov7251_pll_configs {
-	const struct ov7251_pll1_config *pll1;
-	const struct ov7251_pll2_config *pll2;
+/*
+ * Rubbish ordering, but only PLL1 needs to have a separate configuration per
+ * link frequency and the array member needs to be last.
+ */
+struct ov7251_pll_cfgs {
+	const struct ov7251_pll2_cfg *pll2;
+	const struct ov7251_pll1_cfg *pll1[];
+};
+
+enum xclk_rate {
+	OV7251_19_2_MHZ,
+	OV7251_24_MHZ,
+	OV7251_NUM_SUPPORTED_RATES
+};
+
+enum supported_link_freqs {
+	OV7251_LINK_FREQ_240_MHZ,
+	OV7251_LINK_FREQ_319_2_MHZ,
+	OV7251_NUM_SUPPORTED_LINK_FREQS
 };
 
 struct ov7251 {
@@ -130,17 +137,19 @@ struct ov7251 {
 	struct clk *xclk;
 	u32 xclk_freq;
 
-	const struct ov7251_pll_configs *pll_configs;
-
 	struct regulator *io_regulator;
 	struct regulator *core_regulator;
 	struct regulator *analog_regulator;
 
+	const struct ov7251_pll_cfgs *pll_cfgs;
+	enum supported_link_freqs link_freq_idx;
 	const struct ov7251_mode_info *current_mode;
-	const struct ov7251_frame_ival_info *current_ival;
 
 	struct v4l2_ctrl_handler ctrls;
+	struct v4l2_ctrl *pixel_clock;
+	struct v4l2_ctrl *link_freq;
 	struct v4l2_ctrl *exposure;
+	struct v4l2_ctrl *gain;
 	struct v4l2_ctrl *hblank;
 	struct v4l2_ctrl *vblank;
 
@@ -161,57 +170,73 @@ static inline struct ov7251 *to_ov7251(struct v4l2_subdev *sd)
 	return container_of(sd, struct ov7251, sd);
 }
 
-enum xclk_rate {
-	OV7251_19_2_MHZ,
-	OV7251_24_MHZ,
-	OV7251_NUM_SUPPORTED_RATES
-};
-
-static const struct ov7251_pll1_config ov7251_pll1_config_19_2_mhz = {
+static const struct ov7251_pll1_cfg ov7251_pll1_cfg_19_2_mhz_240_mhz = {
 	.pre_div = 0x03,
 	.mult = 0x4b,
 	.div = 0x01,
 	.pix_div = 0x0a,
-	.mipi_div = 0x05
+	.mipi_div = 0x05,
 };
 
-static const struct ov7251_pll1_config ov7251_pll1_config_24_mhz = {
+static const struct ov7251_pll1_cfg ov7251_pll1_cfg_19_2_mhz_319_2_mhz = {
+	.pre_div = 0x01,
+	.mult = 0x85,
+	.div = 0x04,
+	.pix_div = 0x0a,
+	.mipi_div = 0x05,
+};
+
+static const struct ov7251_pll1_cfg ov7251_pll1_cfg_24_mhz_240_mhz = {
 	.pre_div = 0x03,
 	.mult = 0x64,
 	.div = 0x01,
 	.pix_div = 0x0a,
-	.mipi_div = 0x05
+	.mipi_div = 0x05,
 };
 
-static const struct ov7251_pll2_config ov7251_pll2_config_19_2_mhz = {
+static const struct ov7251_pll1_cfg ov7251_pll1_cfg_24_mhz_319_2_mhz = {
+	.pre_div = 0x05,
+	.mult = 0x85,
+	.div = 0x02,
+	.pix_div = 0x0a,
+	.mipi_div = 0x05,
+};
+
+static const struct ov7251_pll2_cfg ov7251_pll2_cfg_19_2_mhz = {
 	.pre_div = 0x04,
 	.mult = 0x32,
 	.div = 0x00,
 	.sys_div = 0x05,
-	.adc_div = 0x04
+	.adc_div = 0x04,
 };
 
-static const struct ov7251_pll2_config ov7251_pll2_config_24_mhz = {
+static const struct ov7251_pll2_cfg ov7251_pll2_cfg_24_mhz = {
 	.pre_div = 0x04,
 	.mult = 0x28,
 	.div = 0x00,
 	.sys_div = 0x05,
-	.adc_div = 0x04
+	.adc_div = 0x04,
 };
 
-static const struct ov7251_pll_configs ov7251_pll_configs_19_2_mhz = {
-	.pll1 = &ov7251_pll1_config_19_2_mhz,
-	.pll2 = &ov7251_pll2_config_19_2_mhz
+static const struct ov7251_pll_cfgs ov7251_pll_cfgs_19_2_mhz = {
+	.pll2 = &ov7251_pll2_cfg_19_2_mhz,
+	.pll1 = {
+		[OV7251_LINK_FREQ_240_MHZ] = &ov7251_pll1_cfg_19_2_mhz_240_mhz,
+		[OV7251_LINK_FREQ_319_2_MHZ] = &ov7251_pll1_cfg_19_2_mhz_319_2_mhz,
+	},
 };
 
-static const struct ov7251_pll_configs ov7251_pll_configs_24_mhz = {
-	.pll1 = &ov7251_pll1_config_24_mhz,
-	.pll2 = &ov7251_pll2_config_24_mhz
+static const struct ov7251_pll_cfgs ov7251_pll_cfgs_24_mhz = {
+	.pll2 = &ov7251_pll2_cfg_24_mhz,
+	.pll1 = {
+		[OV7251_LINK_FREQ_240_MHZ] = &ov7251_pll1_cfg_24_mhz_240_mhz,
+		[OV7251_LINK_FREQ_319_2_MHZ] = &ov7251_pll1_cfg_24_mhz_319_2_mhz,
+	},
 };
 
-static const struct ov7251_pll_configs *ov7251_pll_configs[] = {
-	[OV7251_19_2_MHZ] = &ov7251_pll_configs_19_2_mhz,
-	[OV7251_24_MHZ] = &ov7251_pll_configs_24_mhz
+static const struct ov7251_pll_cfgs *ov7251_pll_cfgs[] = {
+	[OV7251_19_2_MHZ] = &ov7251_pll_cfgs_19_2_mhz,
+	[OV7251_24_MHZ] = &ov7251_pll_cfgs_24_mhz,
 };
 
 static const struct reg_value ov7251_global_init_setting[] = {
@@ -219,7 +244,263 @@ static const struct reg_value ov7251_global_init_setting[] = {
 	{ 0x303b, 0x02 },
 };
 
-static const struct reg_value ov7251_setting_vga[] = {
+static const struct reg_value ov7251_setting_vga_30fps[] = {
+	{ 0x3005, 0x00 },
+	{ 0x3012, 0xc0 },
+	{ 0x3013, 0xd2 },
+	{ 0x3014, 0x04 },
+	{ 0x3016, 0xf0 },
+	{ 0x3017, 0xf0 },
+	{ 0x3018, 0xf0 },
+	{ 0x301a, 0xf0 },
+	{ 0x301b, 0xf0 },
+	{ 0x301c, 0xf0 },
+	{ 0x3023, 0x05 },
+	{ 0x3037, 0xf0 },
+	{ 0x3106, 0xda },
+	{ 0x3503, 0x07 },
+	{ 0x3509, 0x10 },
+	{ 0x3600, 0x1c },
+	{ 0x3602, 0x62 },
+	{ 0x3620, 0xb7 },
+	{ 0x3622, 0x04 },
+	{ 0x3626, 0x21 },
+	{ 0x3627, 0x30 },
+	{ 0x3630, 0x44 },
+	{ 0x3631, 0x35 },
+	{ 0x3634, 0x60 },
+	{ 0x3636, 0x00 },
+	{ 0x3662, 0x01 },
+	{ 0x3663, 0x70 },
+	{ 0x3664, 0x50 },
+	{ 0x3666, 0x0a },
+	{ 0x3669, 0x1a },
+	{ 0x366a, 0x00 },
+	{ 0x366b, 0x50 },
+	{ 0x3673, 0x01 },
+	{ 0x3674, 0xff },
+	{ 0x3675, 0x03 },
+	{ 0x3705, 0xc1 },
+	{ 0x3709, 0x40 },
+	{ 0x373c, 0x08 },
+	{ 0x3742, 0x00 },
+	{ 0x3757, 0xb3 },
+	{ 0x3788, 0x00 },
+	{ 0x37a8, 0x01 },
+	{ 0x37a9, 0xc0 },
+	{ 0x3800, 0x00 },
+	{ 0x3801, 0x04 },
+	{ 0x3802, 0x00 },
+	{ 0x3803, 0x04 },
+	{ 0x3804, 0x02 },
+	{ 0x3805, 0x8b },
+	{ 0x3806, 0x01 },
+	{ 0x3807, 0xeb },
+	{ 0x3808, 0x02 }, /* width high */
+	{ 0x3809, 0x80 }, /* width low */
+	{ 0x380a, 0x01 }, /* height high */
+	{ 0x380b, 0xe0 }, /* height low */
+	{ 0x380c, 0x03 }, /* total horiz timing high */
+	{ 0x380d, 0xa0 }, /* total horiz timing low */
+	{ 0x380e, 0x06 }, /* total vertical timing high */
+	{ 0x380f, 0xbc }, /* total vertical timing low */
+	{ 0x3810, 0x00 },
+	{ 0x3811, 0x04 },
+	{ 0x3812, 0x00 },
+	{ 0x3813, 0x05 },
+	{ 0x3814, 0x11 },
+	{ 0x3815, 0x11 },
+	{ 0x3820, 0x40 },
+	{ 0x3821, 0x00 },
+	{ 0x382f, 0x0e },
+	{ 0x3832, 0x00 },
+	{ 0x3833, 0x05 },
+	{ 0x3834, 0x00 },
+	{ 0x3835, 0x0c },
+	{ 0x3837, 0x00 },
+	{ 0x3b80, 0x00 },
+	{ 0x3b81, 0xa5 },
+	{ 0x3b82, 0x10 },
+	{ 0x3b83, 0x00 },
+	{ 0x3b84, 0x08 },
+	{ 0x3b85, 0x00 },
+	{ 0x3b86, 0x01 },
+	{ 0x3b87, 0x00 },
+	{ 0x3b88, 0x00 },
+	{ 0x3b89, 0x00 },
+	{ 0x3b8a, 0x00 },
+	{ 0x3b8b, 0x05 },
+	{ 0x3b8c, 0x00 },
+	{ 0x3b8d, 0x00 },
+	{ 0x3b8e, 0x00 },
+	{ 0x3b8f, 0x1a },
+	{ 0x3b94, 0x05 },
+	{ 0x3b95, 0xf2 },
+	{ 0x3b96, 0x40 },
+	{ 0x3c00, 0x89 },
+	{ 0x3c01, 0x63 },
+	{ 0x3c02, 0x01 },
+	{ 0x3c03, 0x00 },
+	{ 0x3c04, 0x00 },
+	{ 0x3c05, 0x03 },
+	{ 0x3c06, 0x00 },
+	{ 0x3c07, 0x06 },
+	{ 0x3c0c, 0x01 },
+	{ 0x3c0d, 0xd0 },
+	{ 0x3c0e, 0x02 },
+	{ 0x3c0f, 0x0a },
+	{ 0x4001, 0x42 },
+	{ 0x4004, 0x04 },
+	{ 0x4005, 0x00 },
+	{ 0x404e, 0x01 },
+	{ 0x4300, 0xff },
+	{ 0x4301, 0x00 },
+	{ 0x4315, 0x00 },
+	{ 0x4501, 0x48 },
+	{ 0x4600, 0x00 },
+	{ 0x4601, 0x4e },
+	{ 0x4801, 0x0f },
+	{ 0x4806, 0x0f },
+	{ 0x4819, 0xaa },
+	{ 0x4823, 0x3e },
+	{ 0x4837, 0x19 },
+	{ 0x4a0d, 0x00 },
+	{ 0x4a47, 0x7f },
+	{ 0x4a49, 0xf0 },
+	{ 0x4a4b, 0x30 },
+	{ 0x5000, 0x85 },
+	{ 0x5001, 0x80 },
+};
+
+static const struct reg_value ov7251_setting_vga_60fps[] = {
+	{ 0x3005, 0x00 },
+	{ 0x3012, 0xc0 },
+	{ 0x3013, 0xd2 },
+	{ 0x3014, 0x04 },
+	{ 0x3016, 0x10 },
+	{ 0x3017, 0x00 },
+	{ 0x3018, 0x00 },
+	{ 0x301a, 0x00 },
+	{ 0x301b, 0x00 },
+	{ 0x301c, 0x00 },
+	{ 0x3023, 0x05 },
+	{ 0x3037, 0xf0 },
+	{ 0x3106, 0xda },
+	{ 0x3503, 0x07 },
+	{ 0x3509, 0x10 },
+	{ 0x3600, 0x1c },
+	{ 0x3602, 0x62 },
+	{ 0x3620, 0xb7 },
+	{ 0x3622, 0x04 },
+	{ 0x3626, 0x21 },
+	{ 0x3627, 0x30 },
+	{ 0x3630, 0x44 },
+	{ 0x3631, 0x35 },
+	{ 0x3634, 0x60 },
+	{ 0x3636, 0x00 },
+	{ 0x3662, 0x01 },
+	{ 0x3663, 0x70 },
+	{ 0x3664, 0x50 },
+	{ 0x3666, 0x0a },
+	{ 0x3669, 0x1a },
+	{ 0x366a, 0x00 },
+	{ 0x366b, 0x50 },
+	{ 0x3673, 0x01 },
+	{ 0x3674, 0xff },
+	{ 0x3675, 0x03 },
+	{ 0x3705, 0xc1 },
+	{ 0x3709, 0x40 },
+	{ 0x373c, 0x08 },
+	{ 0x3742, 0x00 },
+	{ 0x3757, 0xb3 },
+	{ 0x3788, 0x00 },
+	{ 0x37a8, 0x01 },
+	{ 0x37a9, 0xc0 },
+	{ 0x3800, 0x00 },
+	{ 0x3801, 0x04 },
+	{ 0x3802, 0x00 },
+	{ 0x3803, 0x04 },
+	{ 0x3804, 0x02 },
+	{ 0x3805, 0x8b },
+	{ 0x3806, 0x01 },
+	{ 0x3807, 0xeb },
+	{ 0x3808, 0x02 }, /* width high */
+	{ 0x3809, 0x80 }, /* width low */
+	{ 0x380a, 0x01 }, /* height high */
+	{ 0x380b, 0xe0 }, /* height low */
+	{ 0x380c, 0x03 }, /* total horiz timing high */
+	{ 0x380d, 0xa0 }, /* total horiz timing low */
+	{ 0x380e, 0x03 }, /* total vertical timing high */
+	{ 0x380f, 0x5c }, /* total vertical timing low */
+	{ 0x3810, 0x00 },
+	{ 0x3811, 0x04 },
+	{ 0x3812, 0x00 },
+	{ 0x3813, 0x05 },
+	{ 0x3814, 0x11 },
+	{ 0x3815, 0x11 },
+	{ 0x3820, 0x40 },
+	{ 0x3821, 0x00 },
+	{ 0x382f, 0x0e },
+	{ 0x3832, 0x00 },
+	{ 0x3833, 0x05 },
+	{ 0x3834, 0x00 },
+	{ 0x3835, 0x0c },
+	{ 0x3837, 0x00 },
+	{ 0x3b80, 0x00 },
+	{ 0x3b81, 0xa5 },
+	{ 0x3b82, 0x10 },
+	{ 0x3b83, 0x00 },
+	{ 0x3b84, 0x08 },
+	{ 0x3b85, 0x00 },
+	{ 0x3b86, 0x01 },
+	{ 0x3b87, 0x00 },
+	{ 0x3b88, 0x00 },
+	{ 0x3b89, 0x00 },
+	{ 0x3b8a, 0x00 },
+	{ 0x3b8b, 0x05 },
+	{ 0x3b8c, 0x00 },
+	{ 0x3b8d, 0x00 },
+	{ 0x3b8e, 0x00 },
+	{ 0x3b8f, 0x1a },
+	{ 0x3b94, 0x05 },
+	{ 0x3b95, 0xf2 },
+	{ 0x3b96, 0x40 },
+	{ 0x3c00, 0x89 },
+	{ 0x3c01, 0x63 },
+	{ 0x3c02, 0x01 },
+	{ 0x3c03, 0x00 },
+	{ 0x3c04, 0x00 },
+	{ 0x3c05, 0x03 },
+	{ 0x3c06, 0x00 },
+	{ 0x3c07, 0x06 },
+	{ 0x3c0c, 0x01 },
+	{ 0x3c0d, 0xd0 },
+	{ 0x3c0e, 0x02 },
+	{ 0x3c0f, 0x0a },
+	{ 0x4001, 0x42 },
+	{ 0x4004, 0x04 },
+	{ 0x4005, 0x00 },
+	{ 0x404e, 0x01 },
+	{ 0x4300, 0xff },
+	{ 0x4301, 0x00 },
+	{ 0x4315, 0x00 },
+	{ 0x4501, 0x48 },
+	{ 0x4600, 0x00 },
+	{ 0x4601, 0x4e },
+	{ 0x4801, 0x0f },
+	{ 0x4806, 0x0f },
+	{ 0x4819, 0xaa },
+	{ 0x4823, 0x3e },
+	{ 0x4837, 0x19 },
+	{ 0x4a0d, 0x00 },
+	{ 0x4a47, 0x7f },
+	{ 0x4a49, 0xf0 },
+	{ 0x4a4b, 0x30 },
+	{ 0x5000, 0x85 },
+	{ 0x5001, 0x80 },
+};
+
+static const struct reg_value ov7251_setting_vga_90fps[] = {
 	{ 0x3005, 0x00 },
 	{ 0x3012, 0xc0 },
 	{ 0x3013, 0xd2 },
@@ -276,6 +557,8 @@ static const struct reg_value ov7251_setting_vga[] = {
 	{ 0x380b, 0xe0 }, /* height low */
 	{ 0x380c, 0x03 }, /* total horiz timing high */
 	{ 0x380d, 0xa0 }, /* total horiz timing low */
+	{ 0x380e, 0x02 }, /* total vertical timing high */
+	{ 0x380f, 0x3c }, /* total vertical timing low */
 	{ 0x3810, 0x00 },
 	{ 0x3811, 0x04 },
 	{ 0x3812, 0x00 },
@@ -359,40 +642,54 @@ static const unsigned long supported_xclk_rates[] = {
 };
 
 static const s64 link_freq[] = {
-	240000000,
+	[OV7251_LINK_FREQ_240_MHZ] = 240000000,
+	[OV7251_LINK_FREQ_319_2_MHZ] = 319200000,
 };
 
-static const struct ov7251_frame_ival_info ov7251_frame_ival_info_data[] = {
-	{
-		.vts = 0x6bc,
-		.timeperframe = {
-			.numerator = 100,
-			.denominator = 3000
-		}
-	},
-	{
-		.vts = 0x35c,
-		.timeperframe = {
-			.numerator = 100,
-			.denominator = 6014
-		}
-	},
-	{
-		.vts = 0x23c,
-		.timeperframe = {
-			.numerator = 100,
-			.denominator = 9043
-		}
-	},
+static const s64 pixel_rates[] = {
+	[OV7251_LINK_FREQ_240_MHZ] = 48000000,
+	[OV7251_LINK_FREQ_319_2_MHZ] = 63840000,
 };
 
 static const struct ov7251_mode_info ov7251_mode_info_data[] = {
 	{
 		.width = 640,
 		.height = 480,
-		.data = ov7251_setting_vga,
-		.data_size = ARRAY_SIZE(ov7251_setting_vga),
+		.vts = 1724,
+		.data = ov7251_setting_vga_30fps,
+		.data_size = ARRAY_SIZE(ov7251_setting_vga_30fps),
+		.exposure_max = 1704,
 		.exposure_def = 504,
+		.timeperframe = {
+			.numerator = 100,
+			.denominator = 3000
+		}
+	},
+	{
+		.width = 640,
+		.height = 480,
+		.vts = 860,
+		.data = ov7251_setting_vga_60fps,
+		.data_size = ARRAY_SIZE(ov7251_setting_vga_60fps),
+		.exposure_max = 840,
+		.exposure_def = 504,
+		.timeperframe = {
+			.numerator = 100,
+			.denominator = 6014
+		}
+	},
+	{
+		.width = 640,
+		.height = 480,
+		.vts = 572,
+		.data = ov7251_setting_vga_90fps,
+		.data_size = ARRAY_SIZE(ov7251_setting_vga_90fps),
+		.exposure_max = 552,
+		.exposure_def = 504,
+		.timeperframe = {
+			.numerator = 100,
+			.denominator = 9043
+		}
 	},
 };
 
@@ -522,32 +819,32 @@ static int ov7251_read_reg(struct ov7251 *ov7251, u16 reg, u8 *val)
 
 static int ov7251_pll_configure(struct ov7251 *ov7251)
 {
-	const struct ov7251_pll_configs *configs;
+	const struct ov7251_pll_cfgs *configs;
 	int ret;
 
-	configs = ov7251->pll_configs;
+	configs = ov7251->pll_cfgs;
 
 	ret = ov7251_write_reg(ov7251, OV7251_PLL1_PRE_DIV_REG,
-			       configs->pll1->pre_div);
+			       configs->pll1[ov7251->link_freq_idx]->pre_div);
 	if (ret < 0)
 		return ret;
 
 	ret = ov7251_write_reg(ov7251, OV7251_PLL1_MULT_REG,
-			       configs->pll1->mult);
+			       configs->pll1[ov7251->link_freq_idx]->mult);
 	if (ret < 0)
 		return ret;
 	ret = ov7251_write_reg(ov7251, OV7251_PLL1_DIVIDER_REG,
-			       configs->pll1->div);
+			       configs->pll1[ov7251->link_freq_idx]->div);
 	if (ret < 0)
 		return ret;
 
 	ret = ov7251_write_reg(ov7251, OV7251_PLL1_PIX_DIV_REG,
-			       configs->pll1->pix_div);
+			       configs->pll1[ov7251->link_freq_idx]->pix_div);
 	if (ret < 0)
 		return ret;
 
 	ret = ov7251_write_reg(ov7251, OV7251_PLL1_MIPI_DIV_REG,
-			       configs->pll1->mipi_div);
+			       configs->pll1[ov7251->link_freq_idx]->mipi_div);
 	if (ret < 0)
 		return ret;
 
@@ -618,8 +915,11 @@ static int ov7251_set_register_array(struct ov7251 *ov7251,
 	return 0;
 }
 
-static int ov7251_set_power_on(struct ov7251 *ov7251)
+static int ov7251_set_power_on(struct device *dev)
 {
+	struct i2c_client *client = container_of(dev, struct i2c_client, dev);
+	struct v4l2_subdev *sd = i2c_get_clientdata(client);
+	struct ov7251 *ov7251 = to_ov7251(sd);
 	int ret;
 	u32 wait_us;
 
@@ -641,34 +941,33 @@ static int ov7251_set_power_on(struct ov7251 *ov7251)
 			       DIV_ROUND_UP(ov7251->xclk_freq, 1000));
 	usleep_range(wait_us, wait_us + 1000);
 
-	return 0;
+	ret = ov7251_set_register_array(ov7251,
+					ov7251_global_init_setting,
+					ARRAY_SIZE(ov7251_global_init_setting));
+	if (ret < 0) {
+		dev_err(ov7251->dev, "error during global init\n");
+		gpiod_set_value_cansleep(ov7251->enable_gpio, 0);
+		clk_disable_unprepare(ov7251->xclk);
+		ov7251_regulators_disable(ov7251);
+		return ret;
+	}
+
+	return ret;
 }
 
-static void ov7251_set_power_off(struct ov7251 *ov7251)
+static int ov7251_set_power_off(struct device *dev)
 {
+	struct i2c_client *client = container_of(dev, struct i2c_client, dev);
+	struct v4l2_subdev *sd = i2c_get_clientdata(client);
+	struct ov7251 *ov7251 = to_ov7251(sd);
+
 	clk_disable_unprepare(ov7251->xclk);
 	gpiod_set_value_cansleep(ov7251->enable_gpio, 0);
 	ov7251_regulators_disable(ov7251);
-}
-
-static int __maybe_unused ov7251_sensor_suspend(struct device *dev)
-{
-	struct v4l2_subdev *sd = dev_get_drvdata(dev);
-	struct ov7251 *ov7251 = to_ov7251(sd);
-
-	ov7251_set_power_off(ov7251);
 
 	return 0;
 }
 
-static int __maybe_unused ov7251_sensor_resume(struct device *dev)
-{
-	struct v4l2_subdev *sd = dev_get_drvdata(dev);
-	struct ov7251 *ov7251 = to_ov7251(sd);
-
-	return ov7251_set_power_on(ov7251);
-}
-
 static int ov7251_set_hflip(struct ov7251 *ov7251, s32 value)
 {
 	u8 val = ov7251->timing_format2;
@@ -703,19 +1002,6 @@ static int ov7251_set_vflip(struct ov7251 *ov7251, s32 value)
 	return ret;
 }
 
-static int ov7251_set_vblank(struct ov7251 *ov7251, s32 value)
-{
-	u16 reg;
-	u8 val[2];
-
-	reg = OV7251_VTS_HIGH;
-	value += ov7251->current_mode->height;
-	val[0] = (value & 0xff00) >> 8; /* goes to OV7251_VTS_HIGH */
-	val[1] = value & 0xff;          /* goes to OV7251_VTS_LOW */
-
-	return ov7251_write_seq_regs(ov7251, reg, val, 2);
-}
-
 static int ov7251_set_test_pattern(struct ov7251 *ov7251, s32 value)
 {
 	u8 val = ov7251->pre_isp_00;
@@ -738,25 +1024,38 @@ static const char * const ov7251_test_pattern_menu[] = {
 	"Vertical Pattern Bars",
 };
 
+static int ov7251_vts_configure(struct ov7251 *ov7251, s32 vblank)
+{
+	u8 vts[2];
+
+	vts[0] = ((ov7251->current_mode->height + vblank) & 0xff00) >> 8;
+	vts[1] = ((ov7251->current_mode->height + vblank) & 0x00ff);
+
+	return ov7251_write_seq_regs(ov7251, OV7251_TIMING_VTS_REG, vts, 2);
+}
+
 static int ov7251_s_ctrl(struct v4l2_ctrl *ctrl)
 {
 	struct ov7251 *ov7251 = container_of(ctrl->handler,
 					     struct ov7251, ctrls);
-	s64 max;
 	int ret;
 
-	/* v4l2_ctrl_lock() locks our mutex */
-	switch (ctrl->id) {
-	case V4L2_CID_VBLANK:
-		/* Update max exposure while meeting expected vblanking */
-		max = ov7251->current_mode->height + ctrl->val - OV7251_EXPOSURE_OFFSET;
+	/* If VBLANK is altered we need to update exposure to compensate */
+	if (ctrl->id == V4L2_CID_VBLANK) {
+		int exposure_max;
+
+		exposure_max = ov7251->current_mode->height + ctrl->val -
+			       OV7251_INTEGRATION_MARGIN;
 		__v4l2_ctrl_modify_range(ov7251->exposure,
-					 ov7251->exposure->minimum, max,
+					 ov7251->exposure->minimum,
+					 exposure_max,
 					 ov7251->exposure->step,
-					 ov7251->exposure->default_value);
-		break;
+					 min(ov7251->exposure->val,
+					     exposure_max));
 	}
 
+	/* v4l2_ctrl_lock() locks our mutex */
+
 	if (!pm_runtime_get_if_in_use(ov7251->dev))
 		return 0;
 
@@ -764,7 +1063,7 @@ static int ov7251_s_ctrl(struct v4l2_ctrl *ctrl)
 	case V4L2_CID_EXPOSURE:
 		ret = ov7251_set_exposure(ov7251, ctrl->val);
 		break;
-	case V4L2_CID_ANALOGUE_GAIN:
+	case V4L2_CID_GAIN:
 		ret = ov7251_set_gain(ov7251, ctrl->val);
 		break;
 	case V4L2_CID_TEST_PATTERN:
@@ -777,7 +1076,7 @@ static int ov7251_s_ctrl(struct v4l2_ctrl *ctrl)
 		ret = ov7251_set_vflip(ov7251, ctrl->val);
 		break;
 	case V4L2_CID_VBLANK:
-		ret = ov7251_set_vblank(ov7251, ctrl->val);
+		ret = ov7251_vts_configure(ov7251, ctrl->val);
 		break;
 	default:
 		ret = -EINVAL;
@@ -830,13 +1129,13 @@ static int ov7251_enum_frame_ival(struct v4l2_subdev *subdev,
 	unsigned int index = fie->index;
 	unsigned int i;
 
-	for (i = 0; i < ARRAY_SIZE(ov7251_frame_ival_info_data); i++) {
-		if (fie->width != ov7251_mode_info_data[0].width ||
-		    fie->height != ov7251_mode_info_data[0].height)
+	for (i = 0; i < ARRAY_SIZE(ov7251_mode_info_data); i++) {
+		if (fie->width != ov7251_mode_info_data[i].width ||
+		    fie->height != ov7251_mode_info_data[i].height)
 			continue;
 
 		if (index-- == 0) {
-			fie->interval = ov7251_frame_ival_info_data[i].timeperframe;
+			fie->interval = ov7251_mode_info_data[i].timeperframe;
 			return 0;
 		}
 	}
@@ -895,18 +1194,23 @@ static inline u32 avg_fps(const struct v4l2_fract *t)
 	return (t->denominator + (t->numerator >> 1)) / t->numerator;
 }
 
-static const struct ov7251_frame_ival_info *
-ov7251_find_frame_ival_by_ival(struct ov7251 *ov7251, struct v4l2_fract *timeperframe)
+static const struct ov7251_mode_info *
+ov7251_find_mode_by_ival(struct ov7251 *ov7251, struct v4l2_fract *timeperframe)
 {
+	const struct ov7251_mode_info *mode = ov7251->current_mode;
 	unsigned int fps_req = avg_fps(timeperframe);
 	unsigned int max_dist_match = (unsigned int) -1;
 	unsigned int i, n = 0;
 
-	for (i = 0; i < ARRAY_SIZE(ov7251_frame_ival_info_data); i++) {
+	for (i = 0; i < ARRAY_SIZE(ov7251_mode_info_data); i++) {
 		unsigned int dist;
 		unsigned int fps_tmp;
 
-		fps_tmp = avg_fps(&ov7251_frame_ival_info_data[i].timeperframe);
+		if (mode->width != ov7251_mode_info_data[i].width ||
+		    mode->height != ov7251_mode_info_data[i].height)
+			continue;
+
+		fps_tmp = avg_fps(&ov7251_mode_info_data[i].timeperframe);
 
 		dist = abs(fps_req - fps_tmp);
 
@@ -916,7 +1220,7 @@ ov7251_find_frame_ival_by_ival(struct ov7251 *ov7251, struct v4l2_fract *timeper
 		}
 	}
 
-	return &ov7251_frame_ival_info_data[n];
+	return &ov7251_mode_info_data[n];
 }
 
 static int ov7251_set_format(struct v4l2_subdev *sd,
@@ -925,9 +1229,9 @@ static int ov7251_set_format(struct v4l2_subdev *sd,
 {
 	struct ov7251 *ov7251 = to_ov7251(sd);
 	struct v4l2_mbus_framefmt *__format;
+	int vblank_max, vblank_def;
 	struct v4l2_rect *__crop;
 	const struct ov7251_mode_info *new_mode;
-	s64 h_blank;
 	int ret = 0;
 
 	mutex_lock(&ov7251->lock);
@@ -944,14 +1248,8 @@ static int ov7251_set_format(struct v4l2_subdev *sd,
 	__crop->height = new_mode->height;
 
 	if (format->which == V4L2_SUBDEV_FORMAT_ACTIVE) {
-		h_blank = OV7251_HTS - new_mode->width;
-		__v4l2_ctrl_modify_range(ov7251->hblank, h_blank,
-					 h_blank, 1, h_blank);
-		__v4l2_ctrl_s_ctrl(ov7251->hblank, h_blank);
-
-		ret = __v4l2_ctrl_modify_range(ov7251->exposure, 1,
-					       ov7251->current_ival->vts -
-							OV7251_EXPOSURE_OFFSET,
+		ret = __v4l2_ctrl_modify_range(ov7251->exposure,
+					       1, new_mode->exposure_max,
 					       1, new_mode->exposure_def);
 		if (ret < 0)
 			goto exit;
@@ -961,6 +1259,18 @@ static int ov7251_set_format(struct v4l2_subdev *sd,
 		if (ret < 0)
 			goto exit;
 
+		ret = __v4l2_ctrl_s_ctrl(ov7251->gain, 16);
+		if (ret < 0)
+			goto exit;
+
+		vblank_max = OV7251_TIMING_MAX_VTS - new_mode->height;
+		vblank_def = new_mode->vts - new_mode->height;
+		ret = __v4l2_ctrl_modify_range(ov7251->vblank,
+					       OV7251_TIMING_MIN_VTS,
+					       vblank_max, 1, vblank_def);
+		if (ret < 0)
+			goto exit;
+
 		ov7251->current_mode = new_mode;
 	}
 
@@ -1008,33 +1318,30 @@ static int ov7251_get_selection(struct v4l2_subdev *sd,
 	struct ov7251 *ov7251 = to_ov7251(sd);
 
 	switch (sel->target) {
+	case V4L2_SEL_TGT_CROP_DEFAULT:
 	case V4L2_SEL_TGT_CROP:
-		mutex_lock(&ov7251->lock);
+	mutex_lock(&ov7251->lock);
 		sel->r = *__ov7251_get_pad_crop(ov7251, sd_state, sel->pad,
 						sel->which);
 		mutex_unlock(&ov7251->lock);
-
-		return 0;
-
+		break;
 	case V4L2_SEL_TGT_NATIVE_SIZE:
 		sel->r.top = 0;
 		sel->r.left = 0;
 		sel->r.width = OV7251_NATIVE_WIDTH;
 		sel->r.height = OV7251_NATIVE_HEIGHT;
-
-		return 0;
-
-	case V4L2_SEL_TGT_CROP_DEFAULT:
+		break;
 	case V4L2_SEL_TGT_CROP_BOUNDS:
-		sel->r.top = OV7251_PIXEL_ARRAY_TOP;
-		sel->r.left = OV7251_PIXEL_ARRAY_LEFT;
-		sel->r.width = OV7251_PIXEL_ARRAY_WIDTH;
-		sel->r.height = OV7251_PIXEL_ARRAY_HEIGHT;
-
-		return 0;
+		sel->r.top = OV7251_ACTIVE_START_TOP;
+		sel->r.left = OV7251_ACTIVE_START_LEFT;
+		sel->r.width = OV7251_ACTIVE_WIDTH;
+		sel->r.height = OV7251_ACTIVE_HEIGHT;
+		break;
+	default:
+		return -EINVAL;
 	}
 
-	return -EINVAL;
+	return 0;
 }
 
 static int ov7251_s_stream(struct v4l2_subdev *subdev, int enable)
@@ -1047,7 +1354,7 @@ static int ov7251_s_stream(struct v4l2_subdev *subdev, int enable)
 	if (enable) {
 		ret = pm_runtime_get_sync(ov7251->dev);
 		if (ret < 0)
-			return ret;
+			goto err_power_down;
 
 		ret = ov7251_set_register_array(ov7251,
 					ov7251_global_init_setting,
@@ -1108,7 +1415,7 @@ static int ov7251_s_stream(struct v4l2_subdev *subdev, int enable)
 	return ret;
 
 err_power_down:
-	pm_runtime_put_noidle(ov7251->dev);
+	pm_runtime_put(ov7251->dev);
 	mutex_unlock(&ov7251->lock);
 	return ret;
 }
@@ -1119,7 +1426,7 @@ static int ov7251_get_frame_interval(struct v4l2_subdev *subdev,
 	struct ov7251 *ov7251 = to_ov7251(subdev);
 
 	mutex_lock(&ov7251->lock);
-	fi->interval = ov7251->current_ival->timeperframe;
+	fi->interval = ov7251->current_mode->timeperframe;
 	mutex_unlock(&ov7251->lock);
 
 	return 0;
@@ -1129,30 +1436,32 @@ static int ov7251_set_frame_interval(struct v4l2_subdev *subdev,
 				     struct v4l2_subdev_frame_interval *fi)
 {
 	struct ov7251 *ov7251 = to_ov7251(subdev);
-	const struct ov7251_frame_ival_info *new_ival;
+	const struct ov7251_mode_info *new_mode;
 	int ret = 0;
 
 	mutex_lock(&ov7251->lock);
-	new_ival = ov7251_find_frame_ival_by_ival(ov7251, &fi->interval);
+	new_mode = ov7251_find_mode_by_ival(ov7251, &fi->interval);
 
-	if (new_ival != ov7251->current_ival) {
-		ret = __v4l2_ctrl_modify_range(ov7251->exposure, 1,
-					       new_ival->vts -
-							OV7251_EXPOSURE_OFFSET,
-					       1, ov7251->exposure->val);
+	if (new_mode != ov7251->current_mode) {
+		ret = __v4l2_ctrl_modify_range(ov7251->exposure,
+					       1, new_mode->exposure_max,
+					       1, new_mode->exposure_def);
 		if (ret < 0)
 			goto exit;
 
-		ret = __v4l2_ctrl_s_ctrl(ov7251->vblank,
-					 new_ival->vts -
-						ov7251->current_mode->height);
+		ret = __v4l2_ctrl_s_ctrl(ov7251->exposure,
+					 new_mode->exposure_def);
 		if (ret < 0)
 			goto exit;
 
-		ov7251->current_ival = new_ival;
+		ret = __v4l2_ctrl_s_ctrl(ov7251->gain, 16);
+		if (ret < 0)
+			goto exit;
+
+		ov7251->current_mode = new_mode;
 	}
 
-	fi->interval = ov7251->current_ival->timeperframe;
+	fi->interval = ov7251->current_mode->timeperframe;
 
 exit:
 	mutex_unlock(&ov7251->lock);
@@ -1188,8 +1497,7 @@ static int ov7251_check_hwcfg(struct ov7251 *ov7251)
 		.bus_type = V4L2_MBUS_CSI2_DPHY,
 	};
 	struct fwnode_handle *endpoint;
-	bool freq_found;
-	int i, j;
+	unsigned int i, j;
 	int ret;
 
 	endpoint = fwnode_graph_get_next_endpoint(fwnode, NULL);
@@ -1202,43 +1510,29 @@ static int ov7251_check_hwcfg(struct ov7251 *ov7251)
 		return dev_err_probe(ov7251->dev, ret,
 				     "parsing endpoint node failed\n");
 
-	if (bus_cfg.bus_type != V4L2_MBUS_CSI2_DPHY) {
-		ret = -EINVAL;
-		dev_err(ov7251->dev, "invalid bus type (%u), must be (%u)\n",
-			bus_cfg.bus_type, V4L2_MBUS_CSI2_DPHY);
-		goto out_free_bus_cfg;
-	}
-
-	if (bus_cfg.bus.mipi_csi2.num_data_lanes != 1) {
-		dev_err(ov7251->dev, "only a 1-lane CSI2 config is supported");
-		ret = -EINVAL;
-		goto out_free_bus_cfg;
-	}
-
 	if (!bus_cfg.nr_of_link_frequencies) {
-		dev_err(ov7251->dev, "no link frequencies defined\n");
-		ret = -EINVAL;
+		ret = dev_err_probe(ov7251->dev, -EINVAL,
+				    "no link frequencies defined\n");
 		goto out_free_bus_cfg;
 	}
 
-	freq_found = false;
 	for (i = 0; i < bus_cfg.nr_of_link_frequencies; i++) {
 		for (j = 0; j < ARRAY_SIZE(link_freq); j++)
-			if (bus_cfg.link_frequencies[i] == link_freq[j]) {
-				freq_found = true;
+			if (bus_cfg.link_frequencies[i] == link_freq[j])
 				break;
-			}
 
-		if (freq_found)
+		if (j < ARRAY_SIZE(link_freq))
 			break;
 	}
 
 	if (i == bus_cfg.nr_of_link_frequencies) {
-		dev_err(ov7251->dev, "no supported link freq found\n");
-		ret = -EINVAL;
+		ret = dev_err_probe(ov7251->dev, -EINVAL,
+				    "no supported link freq found\n");
 		goto out_free_bus_cfg;
 	}
 
+	ov7251->link_freq_idx = i;
+
 out_free_bus_cfg:
 	v4l2_fwnode_endpoint_free(&bus_cfg);
 
@@ -1278,14 +1572,75 @@ static int ov7251_detect_chip(struct ov7251 *ov7251)
 	return 0;
 }
 
+static int ov7251_init_ctrls(struct ov7251 *ov7251)
+{
+	int vblank_max, vblank_def;
+	s64 pixel_rate;
+	int hblank;
+
+	v4l2_ctrl_handler_init(&ov7251->ctrls, 9);
+	ov7251->ctrls.lock = &ov7251->lock;
+
+	v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
+			  V4L2_CID_HFLIP, 0, 1, 1, 0);
+	v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
+			  V4L2_CID_VFLIP, 0, 1, 1, 0);
+	ov7251->exposure = v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
+					     V4L2_CID_EXPOSURE, 1, 32, 1, 32);
+	ov7251->gain = v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
+					 V4L2_CID_GAIN, 16, 1023, 1, 16);
+	v4l2_ctrl_new_std_menu_items(&ov7251->ctrls, &ov7251_ctrl_ops,
+				     V4L2_CID_TEST_PATTERN,
+				     ARRAY_SIZE(ov7251_test_pattern_menu) - 1,
+				     0, 0, ov7251_test_pattern_menu);
+
+	pixel_rate = pixel_rates[ov7251->link_freq_idx];
+	ov7251->pixel_clock = v4l2_ctrl_new_std(&ov7251->ctrls,
+						&ov7251_ctrl_ops,
+						V4L2_CID_PIXEL_RATE,
+						pixel_rate, INT_MAX,
+						pixel_rate, pixel_rate);
+	ov7251->link_freq = v4l2_ctrl_new_int_menu(&ov7251->ctrls,
+						   &ov7251_ctrl_ops,
+						   V4L2_CID_LINK_FREQ,
+						   ARRAY_SIZE(link_freq) - 1,
+						   ov7251->link_freq_idx,
+						   link_freq);
+	if (ov7251->link_freq)
+		ov7251->link_freq->flags |= V4L2_CTRL_FLAG_READ_ONLY;
+	if (ov7251->pixel_clock)
+		ov7251->pixel_clock->flags |= V4L2_CTRL_FLAG_READ_ONLY;
+
+	hblank = OV7251_FIXED_PPL - ov7251->current_mode->width;
+	ov7251->hblank = v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
+					   V4L2_CID_HBLANK, hblank, hblank, 1,
+					   hblank);
+	if (ov7251->hblank)
+		ov7251->hblank->flags |= V4L2_CTRL_FLAG_READ_ONLY;
+
+	vblank_max = OV7251_TIMING_MAX_VTS - ov7251->current_mode->height;
+	vblank_def = ov7251->current_mode->vts - ov7251->current_mode->height;
+	ov7251->vblank = v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
+					   V4L2_CID_VBLANK,
+					   OV7251_TIMING_MIN_VTS, vblank_max, 1,
+					   vblank_def);
+
+	ov7251->sd.ctrl_handler = &ov7251->ctrls;
+
+	if (ov7251->ctrls.error) {
+		v4l2_ctrl_handler_free(&ov7251->ctrls);
+		return ov7251->ctrls.error;
+	}
+
+	return 0;
+}
+
 static int ov7251_probe(struct i2c_client *client)
 {
 	struct v4l2_fwnode_device_properties props;
 	struct device *dev = &client->dev;
-	struct v4l2_ctrl *ctrl;
 	struct ov7251 *ov7251;
-	unsigned int rate = 0;
-	u32 h_blank, v_blank, v_blank_max;
+	unsigned int rate = 0, clk_rate = 0;
 	int ret;
 	int i;
 
@@ -1301,30 +1656,34 @@ static int ov7251_probe(struct i2c_client *client)
 		return ret;
 
 	/* get system clock (xclk) */
-	ov7251->xclk = devm_clk_get(dev, NULL);
-	if (IS_ERR(ov7251->xclk)) {
-		dev_err(dev, "could not get xclk");
-		return PTR_ERR(ov7251->xclk);
-	}
+	ov7251->xclk = devm_clk_get_optional(dev, NULL);
+	if (IS_ERR(ov7251->xclk))
+		return dev_err_probe(dev, PTR_ERR(ov7251->xclk),
+				     "could not get xclk");
 
 	/*
-	 * We could have either a 24MHz or 19.2MHz clock rate from either dt or
+	 * We could have either a 24MHz or 19.2MHz clock rate from either DT or
 	 * ACPI. We also need to support the IPU3 case which will have both an
 	 * external clock AND a clock-frequency property.
 	 */
 	ret = fwnode_property_read_u32(dev_fwnode(dev), "clock-frequency",
 				       &rate);
+	if (ret && !ov7251->xclk)
+		return dev_err_probe(dev, ret, "invalid clock config\n");
+
+	clk_rate = clk_get_rate(ov7251->xclk);
+	ov7251->xclk_freq = clk_rate ? clk_rate : rate;
+
+	if (ov7251->xclk_freq == 0)
+		return dev_err_probe(dev, -EINVAL, "invalid clock frequency\n");
+
 	if (!ret && ov7251->xclk) {
 		ret = clk_set_rate(ov7251->xclk, rate);
 		if (ret)
 			return dev_err_probe(dev, ret,
 					     "failed to set clock rate\n");
-	} else if (ret && !ov7251->xclk) {
-		return dev_err_probe(dev, ret, "invalid clock config\n");
 	}
 
-	ov7251->xclk_freq = rate ? rate : clk_get_rate(ov7251->xclk);
-
 	for (i = 0; i < ARRAY_SIZE(supported_xclk_rates); i++)
 		if (ov7251->xclk_freq == supported_xclk_rates[i])
 			break;
@@ -1334,7 +1693,7 @@ static int ov7251_probe(struct i2c_client *client)
 				     "clock rate %u Hz is unsupported\n",
 				     ov7251->xclk_freq);
 
-	ov7251->pll_configs = ov7251_pll_configs[i];
+	ov7251->pll_cfgs = ov7251_pll_cfgs[i];
 
 	ov7251->io_regulator = devm_regulator_get(dev, "vdddo");
 	if (IS_ERR(ov7251->io_regulator)) {
@@ -1364,53 +1723,10 @@ static int ov7251_probe(struct i2c_client *client)
 	mutex_init(&ov7251->lock);
 
 	ov7251->current_mode = &ov7251_mode_info_data[0];
-	ov7251->current_ival = &ov7251_frame_ival_info_data[0];
-
-	v4l2_ctrl_handler_init(&ov7251->ctrls, 11);
-	ov7251->ctrls.lock = &ov7251->lock;
-
-	v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
-			  V4L2_CID_HFLIP, 0, 1, 1, 0);
-	v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
-			  V4L2_CID_VFLIP, 0, 1, 1, 0);
-	ov7251->exposure = v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
-					     V4L2_CID_EXPOSURE, 1, 32, 1, 32);
-	v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
-			  V4L2_CID_ANALOGUE_GAIN, 16, 1023, 1, 16);
-	v4l2_ctrl_new_std_menu_items(&ov7251->ctrls, &ov7251_ctrl_ops,
-				     V4L2_CID_TEST_PATTERN,
-				     ARRAY_SIZE(ov7251_test_pattern_menu) - 1,
-				     0, 0, ov7251_test_pattern_menu);
-	v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
-			  V4L2_CID_PIXEL_RATE, OV7251_PIXEL_CLOCK,
-			  OV7251_PIXEL_CLOCK, 1, OV7251_PIXEL_CLOCK);
-	ctrl = v4l2_ctrl_new_int_menu(&ov7251->ctrls, &ov7251_ctrl_ops,
-				      V4L2_CID_LINK_FREQ,
-				      ARRAY_SIZE(link_freq) - 1,
-				      0, link_freq);
-	if (ctrl)
-		ctrl->flags |= V4L2_CTRL_FLAG_READ_ONLY;
-	h_blank = OV7251_HTS - ov7251->current_mode->width;
-	ov7251->hblank = v4l2_ctrl_new_std(&ov7251->ctrls, NULL,
-					   V4L2_CID_HBLANK, h_blank,
-					   h_blank, 1, h_blank);
-	if (ov7251->hblank)
-		ov7251->hblank->flags |= V4L2_CTRL_FLAG_READ_ONLY;
-
-	v_blank = ov7251->current_ival->vts - ov7251->current_mode->height;
-	v_blank_max = OV7251_VTS_MAX - ov7251->current_mode->width;
-	ov7251->vblank = v4l2_ctrl_new_std(&ov7251->ctrls, &ov7251_ctrl_ops,
-					   V4L2_CID_VBLANK,
-					   OV7251_VTS_MIN_OFFSET,
-					   v_blank_max, 1, v_blank);
-
-	ov7251->sd.ctrl_handler = &ov7251->ctrls;
-
-	if (ov7251->ctrls.error) {
-		dev_err(dev, "%s: control initialization error %d\n",
-			__func__, ov7251->ctrls.error);
-		ret = ov7251->ctrls.error;
-		goto free_ctrl;
+	ret = ov7251_init_ctrls(ov7251);
+	if (ret) {
+		dev_err_probe(dev, ret, "error during v4l2 ctrl init\n");
+		goto destroy_mutex;
 	}
 
 	ret = v4l2_fwnode_device_parse(&client->dev, &props);
@@ -1434,7 +1750,7 @@ static int ov7251_probe(struct i2c_client *client)
 		goto free_ctrl;
 	}
 
-	ret = ov7251_set_power_on(ov7251);
+	ret = ov7251_set_power_on(ov7251->dev);
 	if (ret)
 		goto free_entity;
 
@@ -1488,17 +1804,18 @@ err_pm_runtime:
 	pm_runtime_disable(ov7251->dev);
 	pm_runtime_put_noidle(ov7251->dev);
 power_down:
-	ov7251_set_power_off(ov7251);
+	ov7251_set_power_off(ov7251->dev);
 free_entity:
 	media_entity_cleanup(&ov7251->sd.entity);
 free_ctrl:
 	v4l2_ctrl_handler_free(&ov7251->ctrls);
+destroy_mutex:
 	mutex_destroy(&ov7251->lock);
 
 	return ret;
 }
 
-static int ov7251_remove(struct i2c_client *client)
+static void ov7251_remove(struct i2c_client *client)
 {
 	struct v4l2_subdev *sd = i2c_get_clientdata(client);
 	struct ov7251 *ov7251 = to_ov7251(sd);
@@ -1510,14 +1827,12 @@ static int ov7251_remove(struct i2c_client *client)
 
 	pm_runtime_disable(ov7251->dev);
 	if (!pm_runtime_status_suspended(ov7251->dev))
-		ov7251_set_power_off(ov7251);
+		ov7251_set_power_off(ov7251->dev);
 	pm_runtime_set_suspended(ov7251->dev);
-
-	return 0;
 }
 
 static const struct dev_pm_ops ov7251_pm_ops = {
-	SET_RUNTIME_PM_OPS(ov7251_sensor_suspend, ov7251_sensor_resume, NULL)
+	SET_RUNTIME_PM_OPS(ov7251_set_power_off, ov7251_set_power_on, NULL)
 };
 
 static const struct of_device_id ov7251_of_match[] = {
