@@ -73,7 +73,6 @@
 #include <media/videobuf2-dma-contig.h>
 
 #include <media/v4l2-async.h>
-#define v4l2_async_notifier_add_subdev __v4l2_async_notifier_add_subdev
 
 #include "vc4-regs-unicam.h"
 
@@ -2079,7 +2078,7 @@ static int unicam_mc_video_link_validate(struct media_link *link)
 	struct v4l2_subdev_format source_fmt;
 	int ret;
 
-	if (!media_entity_remote_pad(link->sink->entity->pads)) {
+	if (!media_entity_remote_source_pad_unique(link->sink->entity)) {
 		unicam_dbg(1, unicam,
 			   "video node %s pad not connected\n", vd->name);
 		return -ENOTCONN;
@@ -2360,7 +2359,7 @@ static void unicam_start_rx(struct unicam_device *dev, dma_addr_t *addr)
 		/* CSI2 */
 		set_field(&val, 1, UNICAM_CLE);
 		set_field(&val, 1, UNICAM_CLLPE);
-		if (dev->bus_flags & V4L2_MBUS_CSI2_CONTINUOUS_CLOCK) {
+		if (!(dev->bus_flags & V4L2_MBUS_CSI2_NONCONTINUOUS_CLOCK)) {
 			set_field(&val, 1, UNICAM_CLTRE);
 			set_field(&val, 1, UNICAM_CLHSE);
 		}
@@ -2382,7 +2381,7 @@ static void unicam_start_rx(struct unicam_device *dev, dma_addr_t *addr)
 		/* CSI2 */
 		set_field(&val, 1, UNICAM_DLE);
 		set_field(&val, 1, UNICAM_DLLPE);
-		if (dev->bus_flags & V4L2_MBUS_CSI2_CONTINUOUS_CLOCK) {
+		if (!(dev->bus_flags & V4L2_MBUS_CSI2_NONCONTINUOUS_CLOCK)) {
 			set_field(&val, 1, UNICAM_DLTRE);
 			set_field(&val, 1, UNICAM_DLHSE);
 		}
@@ -2525,7 +2524,8 @@ static int unicam_start_streaming(struct vb2_queue *vq, unsigned int count)
 		goto err_streaming;
 	}
 
-	ret = media_pipeline_start(&node->video_dev.entity, &node->pipe);
+	ret = media_pipeline_start(dev->node[IMAGE_PAD].video_dev.entity.pads,
+				   &dev->node[IMAGE_PAD].pipe);
 	if (ret < 0) {
 		unicam_err(dev, "Failed to start media pipeline: %d\n", ret);
 		goto err_pm_put;
@@ -2543,9 +2543,7 @@ static int unicam_start_streaming(struct vb2_queue *vq, unsigned int count)
 			goto error_pipeline;
 		}
 
-		dev->active_data_lanes =
-			(mbus_config.flags & V4L2_MBUS_CSI2_LANE_MASK) >>
-					__ffs(V4L2_MBUS_CSI2_LANE_MASK);
+		dev->active_data_lanes = mbus_config.bus.mipi_csi2.num_data_lanes;
 		if (!dev->active_data_lanes)
 			dev->active_data_lanes = dev->max_data_lanes;
 		if (dev->active_data_lanes > dev->max_data_lanes) {
@@ -2621,7 +2619,8 @@ err_vpu_clock:
 		unicam_err(dev, "failed to reset the VPU clock\n");
 	clk_disable_unprepare(dev->vpu_clock);
 error_pipeline:
-	media_pipeline_stop(&node->video_dev.entity);
+	if (node->pad_id == IMAGE_PAD)
+		media_pipeline_stop(dev->node[IMAGE_PAD].video_dev.entity.pads);
 err_pm_put:
 	unicam_runtime_put(dev);
 err_streaming:
@@ -2649,7 +2648,7 @@ static void unicam_stop_streaming(struct vb2_queue *vq)
 
 		unicam_disable(dev);
 
-		media_pipeline_stop(&node->video_dev.entity);
+		media_pipeline_stop(node->video_dev.entity.pads);
 
 		if (dev->clocks_enabled) {
 			if (clk_set_min_rate(dev->vpu_clock, 0))
@@ -2785,7 +2784,7 @@ static void unicam_release(struct kref *kref)
 	media_device_cleanup(&unicam->mdev);
 
 	if (unicam->sensor_state)
-		v4l2_subdev_free_state(unicam->sensor_state);
+		__v4l2_subdev_state_free(unicam->sensor_state);
 
 	kfree(unicam);
 }
@@ -3120,13 +3119,15 @@ static void unregister_nodes(struct unicam_device *unicam)
 
 static int unicam_async_complete(struct v4l2_async_notifier *notifier)
 {
+	static struct lock_class_key key;
 	struct unicam_device *unicam = to_unicam_device(notifier->v4l2_dev);
 	unsigned int i, source_pads = 0;
 	int ret;
 
 	unicam->v4l2_dev.notify = unicam_notify;
 
-	unicam->sensor_state = v4l2_subdev_alloc_state(unicam->sensor);
+	unicam->sensor_state = __v4l2_subdev_state_alloc(unicam->sensor,
+							 "unicam:async->lock", &key);
 	if (!unicam->sensor_state)
 		return -ENOMEM;
 
@@ -3291,18 +3292,18 @@ static int of_unicam_connect_subdevs(struct unicam_device *dev)
 		   dev->max_data_lanes, dev->bus_flags);
 
 	/* Initialize and register the async notifier. */
-	v4l2_async_notifier_init(&dev->notifier);
+	v4l2_async_nf_init(&dev->notifier);
 	dev->notifier.ops = &unicam_async_ops;
 
 	dev->asd.match_type = V4L2_ASYNC_MATCH_FWNODE;
 	dev->asd.match.fwnode = fwnode_graph_get_remote_endpoint(of_fwnode_handle(ep_node));
-	ret = v4l2_async_notifier_add_subdev(&dev->notifier, &dev->asd);
+	ret = __v4l2_async_nf_add_subdev(&dev->notifier, &dev->asd);
 	if (ret) {
 		unicam_err(dev, "Error adding subdevice: %d\n", ret);
 		goto cleanup_exit;
 	}
 
-	ret = v4l2_async_notifier_register(&dev->v4l2_dev, &dev->notifier);
+	ret = v4l2_async_nf_register(&dev->v4l2_dev, &dev->notifier);
 	if (ret) {
 		unicam_err(dev, "Error registering async notifier: %d\n", ret);
 		ret = -EINVAL;
@@ -3439,7 +3440,7 @@ static int unicam_remove(struct platform_device *pdev)
 
 	unicam_dbg(2, unicam, "%s\n", __func__);
 
-	v4l2_async_notifier_unregister(&unicam->notifier);
+	v4l2_async_nf_unregister(&unicam->notifier);
 	v4l2_device_unregister(&unicam->v4l2_dev);
 	media_device_unregister(&unicam->mdev);
 	unregister_nodes(unicam);
