@@ -1321,7 +1321,6 @@ static int bcm2835_mmc_add_host(struct bcm2835_host *host)
 
 		cfg.src_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
 		cfg.dst_addr_width = DMA_SLAVE_BUSWIDTH_4_BYTES;
-		cfg.slave_id = 11;		/* DREQ channel */
 
 		/* Validate the slave configurations */
 
@@ -1475,7 +1474,9 @@ static int bcm2835_mmc_probe(struct platform_device *pdev)
 	}
 
 	if (node) {
-		mmc_of_parse(mmc);
+		ret = mmc_of_parse(mmc);
+		if (ret)
+			goto err;
 
 		/* Read any custom properties */
 		of_property_read_u32(node,
