@@ -626,7 +626,7 @@ static int write_bitstream(struct rpivid_dec_env *const de,
 	// Whether that is the correct behaviour or not is not clear in the
 	// spec.
 	const int rpi_use_emu = 1;
-	unsigned int offset = s->sh->data_bit_offset / 8 + 1;
+	unsigned int offset = s->sh->data_byte_offset;
 	const unsigned int len = (s->sh->bit_size + 7) / 8 - offset;
 	dma_addr_t addr;
 
@@ -781,18 +781,12 @@ static void program_slicecmds(struct rpivid_dec_env *const de,
 // Simply checks POCs
 static int has_backward(const struct v4l2_hevc_dpb_entry *const dpb,
 			const __u8 *const idx, const unsigned int n,
-			const unsigned int cur_poc)
+			const s32 cur_poc)
 {
 	unsigned int i;
 
 	for (i = 0; i < n; ++i) {
-		// Compare mod 2^16
-		// We only get u16 pocs & 8.3.1 says
-		// "The bitstream shall not contain data that result in values
-		//  of DiffPicOrderCnt( picA, picB ) used in the decoding
-		//  process that are not in the range of −2^15 to 2^15 − 1,
-		//  inclusive."
-		if (((cur_poc - dpb[idx[i]].pic_order_cnt[0]) & 0x8000) != 0)
+		if (cur_poc < dpb[idx[i]].pic_order_cnt_val)
 			return 0;
 	}
 	return 1;
@@ -859,11 +853,11 @@ static void pre_slice_decode(struct rpivid_dec_env *const de,
 
 			msg_slice(de,
 				  dpb_no |
-				  (dec->dpb[dpb_no].rps ==
-					V4L2_HEVC_DPB_ENTRY_RPS_LT_CURR ?
+				  ((dec->dpb[dpb_no].flags &
+					V4L2_HEVC_DPB_ENTRY_LONG_TERM_REFERENCE) ?
 						 (1 << 4) : 0) |
 				  (weighted_pred_flag ? (3 << 5) : 0));
-			msg_slice(de, dec->dpb[dpb_no].pic_order_cnt[0]);
+			msg_slice(de, dec->dpb[dpb_no].pic_order_cnt_val & 0xffff);
 
 			if (weighted_pred_flag) {
 				const struct v4l2_hevc_pred_weight_table
@@ -905,11 +899,11 @@ static void pre_slice_decode(struct rpivid_dec_env *const de,
 			//          "L1[%d]=dpb[%d]\n", idx, dpb_no);
 			msg_slice(de,
 				  dpb_no |
-				  (dec->dpb[dpb_no].rps ==
-					 V4L2_HEVC_DPB_ENTRY_RPS_LT_CURR ?
+				  ((dec->dpb[dpb_no].flags &
+					 V4L2_HEVC_DPB_ENTRY_LONG_TERM_REFERENCE) ?
 						 (1 << 4) : 0) |
 					(weighted_pred_flag ? (3 << 5) : 0));
-			msg_slice(de, dec->dpb[dpb_no].pic_order_cnt[0]);
+			msg_slice(de, dec->dpb[dpb_no].pic_order_cnt_val & 0xffff);
 			if (weighted_pred_flag) {
 				const struct v4l2_hevc_pred_weight_table
 					*const w = &sh->pred_weight_table;
@@ -1716,7 +1710,7 @@ static void rpivid_h265_setup(struct rpivid_ctx *ctx, struct rpivid_run *run)
 		default:
 			v4l2_err(&dev->v4l2_dev, "%s: Unexpected state: %d\n",
 				 __func__, de->state);
-		/* FALLTHRU */
+			fallthrough;
 		case RPIVID_DECODE_ERROR_CONTINUE:
 			// Uncleared error - fail now
 			goto fail;
@@ -1918,11 +1912,10 @@ static void rpivid_h265_setup(struct rpivid_ctx *ctx, struct rpivid_run *run)
 				  sh->bit_size, run->src->planes[0].bytesused);
 			goto fail;
 		}
-		if (sh->data_bit_offset >= sh->bit_size ||
-		    sh->bit_size - sh->data_bit_offset < 8) {
+		if (sh->data_byte_offset >= sh->bit_size / 8) {
 			v4l2_warn(&dev->v4l2_dev,
-				  "Bit size %d < Bit offset %d + 8\n",
-				  sh->bit_size, sh->data_bit_offset);
+				  "Bit size %u < Byte offset %u * 8\n",
+				  sh->bit_size, sh->data_byte_offset);
 			goto fail;
 		}
 
@@ -1990,21 +1983,16 @@ static void rpivid_h265_setup(struct rpivid_ctx *ctx, struct rpivid_run *run)
 		goto fail;
 
 	for (i = 0; i < dec->num_active_dpb_entries; ++i) {
-		int buffer_index =
-			vb2_find_timestamp(vq, dec->dpb[i].timestamp, 0);
-		struct vb2_buffer *buf = buffer_index < 0 ?
-					NULL :
-					vb2_get_buffer(vq, buffer_index);
-
+		struct vb2_buffer *buf = vb2_find_buffer(vq, dec->dpb[i].timestamp);
 		if (!buf) {
 			v4l2_warn(&dev->v4l2_dev,
-				  "Missing DPB ent %d, timestamp=%lld, index=%d\n",
-				  i, (long long)dec->dpb[i].timestamp,
-				  buffer_index);
+				  "Missing DPB ent %d, timestamp=%lld\n",
+				  i, (long long)dec->dpb[i].timestamp);
 			continue;
 		}
 
 		if (s->use_aux) {
+			int buffer_index = buf->index;
 			dpb_q_aux[i] = aux_q_ref_idx(ctx, buffer_index);
 			if (!dpb_q_aux[i])
 				v4l2_warn(&dev->v4l2_dev,
